#!/usr/bin/env python

import sys
import subprocess
import os

try:
    import pygtk
    pygtk.require("2.0")
except:
    pass
try:
    import gtk
    import gtk.glade
    import gobject
except:
    sys.exit(1)

INSTALL_DIR = "/usr/share/ltsconf-editor"
os.chdir(INSTALL_DIR)
try:
        import ltsconf
except:
        print "Impossible de trouver la bibliotheque ltsconf"
        sys.exit(1)



FNAME = "/var/lib/tftpboot/ltsp/i386/lts.conf"

def instanciate_lts(fname):
    """Charge le fichier de configuration lts.conf"""
    return ltsconf.LTSCONF(fname)

class guiLtsconf:
    """This is the application"""

    def __init__(self, cstation = None):
        self.store_index = {}
        self.modified = False
        #Set the Glade file
        self.gladefile = "ltsconf.glade"
        self.wTree = gtk.glade.XML(self.gladefile, "window1")
        window = self.wTree.get_widget('window1')
        self.cmac = ""
        #Create our dictionay and connect it
        dic = {"on_window1_destroy" : self.exitApp, 
               "on_removeStation_clicked" : self.remove_station_selected,
               "on_writeFile_clicked" : self.write_cfgfile,
               "on_stations_row_activated" : self.add_station,
               "on_onQuit_clicked" : self.exitApp,
               "on_addStation_clicked" : self.add_new_station}
        self.wTree.signal_autoconnect(dic)
        self.initialize_lts()
        self.tree = self.wTree.get_widget('stations')
        self.add_station_column("NOM", 0)
        self.station_list = gtk.TreeStore(gobject.TYPE_STRING)
        #Attach the model to the treeView
        self.tree.set_model(self.station_list)
        self._list_stations()
        window.show_all() 
        if cstation is not None:
            my_res = self.search(self.station_list, self.match_func, (0, cstation))
            if my_res is not None:
                self.add_station(None, my_res.path)
            else:
                self.cmac = cstation

    def match_func(self, row, data):
        column, key = data # data is a tuple containing column number, key
        return row[column] == key

    def search(self, rows, func, data):
        if not rows: 
            return None
        for row in rows:
            if func(row, data):
                return row
            result = self.search(row.iterchildren(), func, data)
            if result: 
                return result.get_iter()
        return None

    def initialize_lts(self,fname = FNAME):
        try:
            self.fname = fname
            self.ltsconf = instanciate_lts(self.fname)
        except:
            errorMessage("Le fichier %s n'est pas accessible\nVerifier que le fichier existe"% self.fname)
            sys.exit(1)

    def add_new_station(self, name):
        selection = self.tree.get_selection()
        selection.unselect_all()
        self.add_station(None)

    def add_station(self, name, item = None, item2 = None):
        #item is the station to update
        station_path = None
        if item is not None:
            station_dlg = Station(station=self.ltsconf.get_station(self.station_list[item][0]), base = self.ltsconf)
        else:
            station_dlg = Station(base = self.ltsconf, mac = self.cmac);
        result, new_station  = station_dlg.run()
        if result == 0:
            self.ltsconf.remove_station(new_station['MAC'])
            if item != None:
                self.station_list.remove(self.station_list.get_iter(item))
            self.insert_row(self.station_list,None, new_station['MAC'])    
            self.ltsconf.add_station(new_station)
            self.modified = True
        else:
            return
        
    def insert_row(self,model,parent,firstcolumn):
        myiter=model.insert_after(parent, None)
        model.set_value(myiter, 0, firstcolumn)
        return myiter
    
    def remove_station_selected(self, name, item = None, item1 = None):
        selection = self.tree.get_selection()
        store, iter_sel  = selection.get_selected()
        self.ltsconf.remove_station(self.station_list[iter_sel][0])
        self.station_list.remove(iter_sel)
        self.modified = True

    def add_station_column(self, name, nb ):
        column = gtk.TreeViewColumn(name, gtk.CellRendererText(),text=nb)
        column.set_resizable(True)
        self.tree.append_column(column)

    def _list_stations(self):
        for station in self.ltsconf.list_stations():
            if self.ltsconf.get_station(station).has_key('LIKE'):
                modele = self.ltsconf.get_station(station)['LIKE']
                self.insert_row(self.station_list, None, station)
            else:
                self.insert_row(self.station_list, None, station)

    def write_cfgfile(self, kwargs):
        self.modified = False
        self.ltsconf.write_file()

    def exitApp(self, name):
        if self.modified == True:
            response = self.warn_unsaved_file(u"Il y a des modifications non enregistrees\nSouhaitez vous enregistrer les modifications?")  
            if response == False :
                pass
            else:
                gtk.main_quit()
        else:
            gtk.main_quit()
    
    def warn_unsaved_file(self, message):
        (RESPONSE_SAVE, RESPONSE_DISCARD) = (gtk.RESPONSE_YES, gtk.RESPONSE_NO)
        dialog = gtk.MessageDialog(None,
                        gtk.DIALOG_MODAL,
                        gtk.MESSAGE_QUESTION,
                        gtk.BUTTONS_NONE,
                        message)
        dialog.add_buttons(gtk.STOCK_SAVE, RESPONSE_SAVE)
        dialog.add_buttons(gtk.STOCK_NO, RESPONSE_DISCARD)
        dialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dialog.set_default_response(RESPONSE_SAVE)
        response = dialog.run()
        dialog.destroy()
        if response == RESPONSE_DISCARD:
            return True
        elif response in [gtk.RESPONSE_CANCEL, gtk.RESPONSE_DELETE_EVENT]:
            return False
        elif response == RESPONSE_SAVE:
            print self.modified
            if self.modified == True:
                self.write_cfgfile(self)
                self.modified = False
                return True


class Station():
    """All properties for a thin-client"""

    def __init__(self,station = None, base = None, mac = None): 
        if station is not None:
            self.station = station
            self.cmac = self.station['MAC']
        elif (station is None and mac is not None):
            self.station = None
            self.cmac = mac
        self.base=base
        self.gladefile = "ltsconf.glade"
        
    def run(self ):
        self.options = {'MAC':None,
                      'DISABLE_X11VNC':None,
                      'LDM_USERNAME':None,
                      'LDM_PASSWORD':None,
                      'LDM_AUTOLOGIN':None,
                      'LDM_DIRECTX':'False',
                      'XSERVER':None,
                      'X_MODE_0':None,
                      'X_MODE_1':None,
                      'X_MODE_2':None,
                      'X_VERTREFRESH':None,
                      'X_HORZSYNC':None,
                      'X_COLOR_DEPTH':None,
                      'X_MOUSE_PROTOCOL':None,
                      'X_MOUSE_DEVICE':None,
                      'X_MOUSE_BUTTONS':None,
                      'X_MOUSE_EMULATE3BTN':None,
                      'LIKE':None
                      }
        self.wTree = gtk.glade.XML(self.gladefile, "dialog1")
        self.dlg = self.wTree.get_widget("dialog1")
        #On remplit avec les valeurs deja connues:
        self.myW = self.wTree.get_widget("MAC")
        self.myW.set_text(self.cmac)
        self.myW.grab_focus()
        actions ={'on_button2_clicked':lambda w:self.verified(None),
                  'on_button1_clicked':lambda w:self.dlg.response(gtk.RESPONSE_CANCEL),
                  'on_lastconnected_changed':lambda w:self.select_station()
                    }
        self.wTree.signal_autoconnect(actions)

        self.fill_models()
        self.get_last_connected()
        # Fill gui with known values
        if self.station is not None:
            for option_set in self.station.keys():
                current_widget = self.wTree.get_widget(option_set)
                if isinstance(current_widget,gtk.Entry):
                    current_widget.set_text(self.station[option_set])
                elif isinstance(current_widget, gtk.ComboBox):
                    if self.station.has_key(option_set):
                        for ele in range(0, len(current_widget.get_model())):
                            if self.station[option_set] == current_widget.get_model()[ele][0]:
                                current_widget.set_active(ele)

        self.result = self.dlg.run()
        #Get values from gui
        for option in self.options.keys():
            current_widget = self.wTree.get_widget(option)
            if isinstance(current_widget,gtk.Entry):
                self.options[option] = current_widget.get_text()
            elif isinstance(current_widget, gtk.ComboBox):
                self.options[option] = current_widget.get_active_text()
    
        self.dlg.destroy()
        return self.result,self.options
    
    def select_station(self):
        choice = self.wTree.get_widget('lastconnected').get_active_text()
        if choice is not None:
            self.wTree.get_widget('MAC').set_text(choice)
    
    def get_last_connected(self):
        cmd = "cat /var/log/syslog |grep DHCPACK |grep \"`export LC_ALL=C ;date +%b' '%d`\" |awk '{print $10}'|sort -u"
        last_connected = subprocess.Popen([cmd], shell=True, stdout=subprocess.PIPE).stdout.read().split()
        model_scrit = gtk.ListStore(str)
        model_scrit.append([None])
        list_models = self.wTree.get_widget("lastconnected") 
        list_models.set_model(model_scrit)
        cell = gtk.CellRendererText()
        list_models.pack_start(cell, True)
        list_models.add_attribute(cell, 'text', 0)
        if len(last_connected) > 0:
            for station in last_connected:
    	        if ":" in station:        
            	    model_scrit.append([station])

    def fill_models(self):
        model_scrit = gtk.ListStore(str)
        model_scrit.append([None])
        list_models = self.wTree.get_widget("LIKE") 
        list_models.set_model(model_scrit)
        cell = gtk.CellRendererText()
        list_models.pack_start(cell, True)
        list_models.add_attribute(cell, 'text', 0)
        for station in self.base.list_stations():
            if (self.station is not None) and (self.station['MAC'] != station ):
                model_scrit.append([station])
            elif self.station is None:
                model_scrit.append([station])

    def verified(self, name):
        if self.myW.get_text() == "":
            errorMessage('Il faut renseigner ce champs')
            self.myW.set_text('')
            self.wTree.get_widget('notebookProperties2').set_current_page(0)
            self.myW.grab_focus()
            self.dlg.run()
        elif (self.base.get_station(self.myW.get_text())) and (self.station is None):
            errorMessage('Cette station existe deja')
            self.myW.set_text('')
            self.myW.grab_focus()
            self.dlg.run()

class errorMessage():
    """Une classe pour gerer les erreurs et afficher un message"""
    
    def __init__(self, message):
        self.show_warning(message)

    def show_warning(self, message): 
        message = gtk.MessageDialog(None,  gtk.DIALOG_MODAL,  gtk.MESSAGE_ERROR,  gtk.BUTTONS_NONE,  message)
        message.add_button(gtk.STOCK_QUIT,  gtk.RESPONSE_CLOSE)
        resp = message.run()
        if resp  ==  gtk.RESPONSE_CLOSE:
            message.destroy()
            return False

if __name__ == "__main__":
    if os.getuid() != 0:
	   errorMessage("Le programme doit s'excuter avec les droits administrateur")
	   sys.exit(1)    
    if os.getenv('LTSP_MAC') is not None:
        ltsconf = guiLtsconf(os.getenv('LTSP_MAC'))
    else:
        ltsconf = guiLtsconf('N')
    gtk.main()
