#!/usr/bin/env python

import sys

class LTSCONF:

    def __init__(self, cfgfile):
        self.storeig_file = cfgfile
        self.store = {}
        self.open_file()
        self.raw_config = ""

    def open_file(self):
        try:
            input_file = open(self.storeig_file,'r')
            self.raw_config = input_file.readlines()
            input_file.close()
            self._read_config()
        except Exception:
            raise Exception

    def _read_config(self):
        """Lire un fichier de configuration"""

        for lines in self.raw_config:
            line = lines.lstrip()
            if line:
                if line[0] == "[" :
                    #A section
                    sec = line[1:len(line)-2].strip()
                    self.store[sec]={}
                elif line[0].strip() == '\n' or line[0].strip() == '#':
                    #These lines are comments
                    continue
                else:
                    #Keys and Values for current section 
                    tab=line.strip().replace(' ','',line.count(' ')).split('=')
                    self.store[sec][tab[0]]=tab[1]

    def get_config(self):   
        return(self.store)

    def write_file(self):
        self._remove_empty_options()
        try : 
            fd = open(self.storeig_file,'w')
            for section in self.store.keys():
                fd.write("["+section+"]\n")
                for option in self.store[section].keys():
                    fd.write(option+"="+self.store[section][option]+"\n")
                fd.write("\n")
            fd.close()
        except:
            sys.exit("Impossible d'enregistrer le fichier de configuration")

    def list_stations(self):
        return self.store.keys()

    def list_groups(self):
        groups = [] 
        for station in self.store.keys():
            if self.store[station].has_key('LIKE'):
                groups.append(self.store[station]['LIKE'])
            else:
                pass
        return groups 

    def get_station(self, station):
        if self.store.has_key(station):
            return self.store[station]
        else:
            return None

    def remove_station(self, station):
        if self.store.has_key(station):
            self.store.pop(station)

    def add_station(self, station):
        self.store[station['MAC']]=station

    def add_group(self, group):
        if self.store.has_key(group):
            pass
        else:
            self.store[group]=[]

    def get_option(self, station, option):
        if self.store.has_key(station):
            if self.store[station].has_key(option):
                return self.store[station][option]
            else:
                print "Pas d'option %s"%option
        else:
            print "Pas de station %s"%station

    def set_option(self, station, option, value):
        if self.store.has_key(station):
            self.store[station][option]=value

    def add_station_to_group(self, station, group):
        if self.store.has_key(station):
            if self.store.has_key(group):
                self.store[station]['LIKE']=group
            else:
                print "Le groupe est inconnu"

    def update_station(self, station):
        if self.store.has_key(station):
            self.store.remove(station)
            self.store.add_station(station)

    def _remove_empty_options(self):
        for station in self.store.keys():
            for option in self.store[station].keys():
                if (self.store[station][option] == None) or (self.store[station][option] == ""):
                    self.store[station].pop(option)
